*~~~~~~~~~~~~ Define column and row target values ~~~~~~~~~~~~~~~~~~~~~~~

$ontext
    The row and column targets are defined in this file.

    The code offers 4 options - 3 are based on the prior sam
 while the fourth requires data from the Excel workbook.
 
   IF BETTER PRIOR INFORMATION ABOUT COLUMN SUMS IS KNOWN, USE IT.
   
    There are 5 steps.
  
  1. Check the differences between the totals of the rows and columns
     are not excessive.
     
  2. If opt_RC_exog assign row and column total exogenously from worksheet
     Row-Col: this will over write the values for colsum0(sac) and
              rowsum0(sac)
              
  3. Assign targets based on Row and column controls (rowsum0 and colsum0)
     The choice is controlled by the parameter target.
      Target = 1 - target totals are mean of prior Sam row & column totals
      Target = 2 - target totals are protosam column totals
      Target = 3 - target totals are protosam row totals 

  4. Assign Targets for blocks specified as coefficients
       If row-colum totals are flagged in CMAC(ssn,ssnp), then specify
       the targets for the corresponding blocks of cells defined by
       map_ss_sac(ssn,sacn).
        This option requires 'opt_blocks' is set to 1.

  5. Assign Targets for blocks specified as coefficients
       IF opt_tv = 1, set ivalue and acell for particular SAM
       values depending on SAMOPT_TV values.     

$offtext

*~~~~~ Check for Large Differences between totals for P_SAM and Exog totals
$onText
  This sub section compares the totals for rows and columns in
     1.  P_SAM - p_colsum and p_rowsum
     2.  The EXOGENOUS values from Excel - rowcol_tot('colsum1'/'rowsum1') 

$offtext


 Parameter
   rowcolchk    CHECK on differences between row and column totals
 ;

LOOP{sacn,
* sac_chk(sacn) = yes ;
 
 rowcolchk = p_colsum(sacn) - rowcol_tot(sacn,"colsum1")  ;
 
  IF[{rowcol_tot(sacn,"colsum1") and [ABS(rowcolchk) gt 0.25 * ABS(p_colsum(sacn) ) ] },
   display "### ERROR CHECK ### Warning. Large discrepancy in rowcol_tot value",
*           sac_chk, rowcolchk ;
            rowcolchk ;
*end if
  ] ;
  
 rowcolchk = p_rowsum(sacn) - rowcol_tot(sacn,"rowsum1")  ;
 
  IF[{rowcol_tot(sacn,"rowsum1") and [ABS(rowcolchk) gt 0.25 * ABS(p_rowsum(sacn) ) ] },
   display "### ERROR CHECK ### Warning. Large discrepancy in rowcol_tot value",
*           sac_chk, rowcolchk ;
             rowcolchk ;
*end if
  ] ;

*end loop
 } ;
 
*~~~~~~~ Assign row and colum totals FOR ESTIMATION

*~~~ apply EXOGENOUS controls if opt_RC_exog = 0

if(opt_RC_exog = 0 ,
 colsum0(sacn)$rowcol_tot(sacn,"colsum1") = rowcol_tot(sacn,"colsum1");
 rowsum0(sacn)$rowcol_tot(sacn,"rowsum1") = rowcol_tot(sacn,"rowsum1");
 ) ;
 
*~~~ apply ENDOGENOUS controls if opt_RC_exog = 1

if(opt_RC_exog = 1 ,
 colsum0(sacn) = p_colsum(sacn) ;
 rowsum0(sacn) = p_rowsum(sacn) ;
 ) ;

*~~~~~ Assigning targets based on Row and column controls (rowsum0 and colsum0)

*~~~ check target selection legitimate
 IF(((target lt 1) or (target gt 3)),
   Display "### ERROR CHECK ### Error in target ###", target ;
   target = 1 ;
 ) ;

IF{target = 1 ,
  coltarget(icol)  = colsum0(icol) ;
  coltarget(sacbal) = 0.50*[colsum0(sacbal) + rowsum0(sacbal)] ;

  rowtarget(irow)  = rowsum0(irow) ;
  rowtarget(sacbal) = 0.50*[colsum0(sacbal) + rowsum0(sacbal)] ;
*End if
  } ;

IF{target = 2 ,
  coltarget(icol)  = colsum0(icol) ;
  coltarget(sacbal) = colsum0(sacbal) ;

  rowtarget(irow)  = rowsum0(irow) ;
  rowtarget(sacbal) = colsum0(sacbal) ;
*End if
  } ;

IF{target = 3 ,
  coltarget(icol)  = colsum0(icol) ;
  coltarget(sacbal) = rowsum0(sacbal) ;

  rowtarget(irow)  = rowsum0(irow) ;
  rowtarget(sacbal) = rowsum0(sacbal) ;
*End if
  } ;
  
*~~~~~~ Assign Targets for blocks specified as coefficients

*  NOTE: Only use this option if opt_blocks is set to 1.

*~~ Assign sig1mac for COLUMNS
 IF{(opt_blocks eq 1),
  LOOP[ssn$(cmac("m_sam_tot",ssn) or sigmac("m_sam_tot",ssn)),
   LOOP((irow,icol)$(map_ss_sac(ssn,icol) and map_ss_sac(ssn,irow)),
     IF(cmac("m_sam_tot",ssn),
       coltarget(icol)  = colsum0(icol) ;
       coltarget(sacbal) = colsum0(sacbal) ;

       rowtarget(irow)  = rowsum0(irow) ;
       rowtarget(sacbal) = colsum0(sacbal) ;
     ) ;
      sig1mac(icol)$sigmac("m_sam_tot",ssn) = sigmac("m_sam_tot",ssn) ;
    ) ;
   ] ;

*~~ Assign sig1mac for ROWS

  LOOP[ssn$(cmac("m_sam_tot",ssn) or sigmac("m_sam_tot",ssn)),
   LOOP((irow,icol)$(map_ss_sac(ssn,icol) and map_ss_sac(ssn,irow)),
     IF(cmac("m_sam_tot",ssn),
       coltarget(icol)  = colsum0(icol) ;
       coltarget(sacbal) = rowsum0(sacbal) ;

       rowtarget(irow)  = rowsum0(irow) ;
       rowtarget(sacbal) = rowsum0(sacbal) ;
     ) ;
      sig1mac(irow)$sigmac(ssn,"m_sam_tot") = sigmac(ssn,"m_sam_tot") ;
    ) ;
   ] ;
   
*~~ Assign sig1mac for COLUMNS

  LOOP[ssn$(((cmac(ssn,"m_sam_tot") and cmac("m_sam_tot",ssn)))
       or sigmac("m_sam_tot",ssn)),
   LOOP((irow,icol)$(map_ss_sac(ssn,icol) and map_ss_sac(ssn,irow)),
    IF((cmac(ssn,"m_sam_tot") and cmac("m_sam_tot",ssn)),
      coltarget(icol)  = colsum0(icol) ;
      coltarget(sacbal) = 0.50*(colsum0(sacbal) + rowsum0(sacbal)) ;

      rowtarget(irow)  = rowsum0(irow) ;
      rowtarget(sacbal) = 0.50*(colsum0(sacbal) + rowsum0(sacbal)) ;
    ) ;
     sig1mac(icol)$(sigmac("m_sam_tot",ssn) or sigmac(ssn,"m_sam_tot")) =
        MAX(sigmac("m_sam_tot",ssn),sigmac(ssn,"m_sam_tot")) ;
    ) ;
   ] ;
*end IF for opt_blocks and CMAC
 };
 
*~~~~~~~Assign Targets for blocks specified as coefficients

* Set ivalue and acell for particular SAM values if
*       opt_tv = 1, depending on SAMOPT_TV values.

 IF{opt_tv eq 1,
  ivalue(irow,icol)$[(SAMOPT_TV(irow,icol,"code_tv") eq 0.0 ) and
                      SUM(nsam, ABS(SAMOPT_TV(irow,icol,nsam)))]  = YES ;
  
  ivalue(irow,icol)$(SAMOPT_TV(irow,icol,"code_tv") eq 1)   = NO  ;
  ivalue(irow,icol)$(SAMOPT_TV(irow,icol,"code_tv") eq 2)   = YES ;
  acell(irow,icol)$(SAMOPT_TV(irow,icol,"code_tv")  eq 2)   = NO  ;
*End if
 } ;
 
*~~~~~~~~~~ END OF DEFINE COLUMN AND ROW TARGET VALUES ~~~~~~~~~~~~~~~