*~~~~~~~~~~~~~~~~~ 17. PRELIMINARY REPORT ON RESULTS ~~~~~~~~~~~~~~~~~~~

* --------- Sets for reporting results

SET pctchg  Boundaries for percentage changes /
   p1    1 percent
   p2    2 percent
   p3    3 percent
   p5    5 percent
   p10   10 percent
   p15   15 percent
   p25   25 percent
   p50   50 percent
   p100  100 percent
   / ;
SET levchg  Boundaries for level changes /
   l1    1 percent of SAMCEMEAN plus 2xSAMCESTDDEV
   l2    2 percent of SAMCEMEAN plus 2xSAMCESTDDEV
   l3    3 percent of SAMCEMEAN plus 2xSAMCESTDDEV
   l5    5 percent of SAMCEMEAN plus 2xSAMCESTDDEV
   l10   10 percent of SAMCEMEAN plus 2xSAMCESTDDEV
   l15   15 percent of SAMCEMEAN plus 2xSAMCESTDDEV
   l25   25 percent of SAMCEMEAN plus 2xSAMCESTDDEV
   l50   50 percent of SAMCEMEAN plus 2xSAMCESTDDEV
   l100  100 percent of SAMCEMEAN plus 2xSAMCESTDDEV
   / ;


*---------------- Parameters for reporting results
Parameters
 SAMCE(sac,sacp)      New estimated SAM flows from CE
 SEM                  Standard deviation of coefficient changes
 VALDIFF(sac,sacp)    Differnce btw original SAM and Final SAM in values
 PERDIFF(sac,sacp)    Differnce btw original SAM and Final SAM in Percent
 bigdiffv(sac,sacp)   Cells with large value changes
 bigdiffp(sac,sacp)   Cells with large percent change
 bigdiffvp(sac,sacp)  Cells with large value AND percent change
 NormEntrop           Normalized Entropy a measure of total uncertainty

* SAMCE descriptive statistics
 SAMCENONZCNT         Number of nonzero elements in SAMCE
 SAMCEMEAN            Mean of nonzero elements in SAMCE
 SAMCEMAX             MAX of elements in SAMCE
 SAMCEMIN             MIN of elements in SAMCE
 SAMCESTDEV           Standard deviation of nonzero elements in SAMCE
 SAMCE1DEVLOCNT       No of nonzero elements less than mean - 1 stddev in SAMCE
 SAMCE1DEVHICNT       No of nonzero elements greater than mean + 1 stddev in SAMCE
 SAMCE2DEVLOCNT       No of nonzero elements less than mean - 2 stddev in SAMCE
 SAMCE2DEVHICNT       No of nonzero elements greater than mean + 2 stddev in SAMCE
 
* Big difference parameters

nbiggdiff(sac,sacp,pctchg,levchg)  Cells with large percent and value changes
biggdiffcnt(pctchg,levchg)         Count of conforming cells in each configuration
chgintpct(pctchg)                  Interval boundary values for pct changes
chgintlev(levchg)                  Interval boundary values for lev changes

  ;


 SAMCE(irow,icol)        = SAM.L(irow,icol);
 SAMCE(t_tot,icol)     = SUM(irow, SAMCE(irow,icol)) ;
 SAMCE(irow,t_tot)     = SUM(icol, SAMCE(irow,icol)) ;
 SAMBALCHK(sacbal)       = SAMCE('TOTAL',sacbal) - SAMCE(sacbal,'TOTAL');


*---- Compute descriptive statistics for final SAMCE ----------------------



* --- Compute basic descriptive stats for SAMCE

* They are interesting in themselves but also useful for determining big diffs
* below. Similar descriptive stats are calculated for the SAM prior to
* estimation above, a comparison could be useful.

 SAMCENONZCNT    = SUM((sacn,sacnp)$SAMCE(sacn,sacnp),1) ;

 SEM             = SQRT(Sum((irow,icol),
                   SQR(COEFF.L(irow,icol) - coeff0(irow,icol)))/SAMCENONZCNT) ;
                   
*SEM = Sum((irow,icol), SQR(COEFF.L(irow,icol) - coeff0(irow,icol)))/SQR(9) ;

 SAMCEMEAN       = SUM((sacn,sacnp), SAMCE(sacn,sacnp))/SAMCENONZCNT ;

 SAMCEMAX        = SMAX((sacn,sacnp), SAMCE(sacn,sacnp)) ;

 SAMCEMIN        = SMIN((sacn,sacnp), SAMCE(sacn,sacnp)) ;

 SAMCESTDEV      = SQRT(SUM((sacn,sacnp)$SAMCE(sacn,sacnp),
                   POWER((SAMCE(sacn,sacnp)-SAMCEMEAN),2))/ SAMCENONZCNT) ;

 SAMCE1DEVLOCNT  = SUM((sacn,sacnp)$(SAMCE(sacn,sacnp) AND SAMCE(sacn,sacnp)
                      < (SAMCEMEAN - SAMCESTDEV)), 1) ;

 SAMCE1DEVHICNT  = SUM((sacn,sacnp)$(SAMCE(sacn,sacnp) AND SAMCE(sacn,sacnp)
                      > (SAMCEMEAN + SAMCESTDEV)), 1) ;

 SAMCE2DEVLOCNT  = SUM((sacn,sacnp)$(P_SAM(sacn,sacnp) AND SAMCE(sacn,sacnp)
                      < (SAMCEMEAN - (2 * SAMCESTDEV))), 1) ;

 SAMCE2DEVHICNT  = SUM((sacn,sacnp)$(P_SAM0(sacn,sacnp) AND SAMCE(sacn,sacnp)
                      > (SAMCEMEAN + (2 * SAMCESTDEV))), 1) ;

 valdiff(sac,sacp)               = samce(sac,sacp) - P_SAM0(sac,sacp) ;
 perdiff(sac,sacp)$P_SAM0(sac,sacp) = 100*(samce(sac,sacp)/P_SAM0(sac,sacp) - 1 ) ;

 bigdiffv(sac,sacp)$(abs(valdiff(sac,sacp)) gt 10) = valdiff(sac,sacp) ;
 bigdiffp(sac,sacp)$(abs(perdiff(sac,sacp)) gt 25) = perdiff(sac,sacp) ;

 bigdiffvp(sac,sacp)$(bigdiffv(sac,sacp) AND  bigdiffp(sac,sacp)) = yes ;

* SAM big level and percentage differences

* Assign boundary values
 chgintpct('p1')      = 1 ;
 chgintpct('p2')      = 2 ;
 chgintpct('p3')      = 3 ;
 chgintpct('p5')      = 5 ;
 chgintpct('p10')     = 10 ;
 chgintpct('p15')     = 15 ;
 chgintpct('p25')     = 25 ;
 chgintpct('p50')     = 50 ;
 chgintpct('p100')    = 100 ;

* For levels, first assign plain values then adjust them so that they are
* proportional to SAMCEMEAN+2xSAMCESTDEV

 chgintlev('l1')      = 1 ;
 chgintlev('l2')      = 2 ;
 chgintlev('l3')      = 3 ;
 chgintlev('l5')      = 5 ;
 chgintlev('l10')     = 10 ;
 chgintlev('l15')     = 15 ;
 chgintlev('l25')     = 25 ;
 chgintlev('l50')     = 50 ;
 chgintlev('l100')    = 100 ;

 chgintlev(levchg) = (chgintlev(levchg)/100) * (SAMCEMEAN+(2*SAMCESTDEV)) ;

* Calculating matrix of level and percentage change counts

 nbiggdiff(sac,sacp,pctchg,levchg)${[abs(perdiff(sac,sacp)) gt chgintpct(pctchg)]
                                AND [abs(valdiff(sac,sacp)) gt chgintlev(levchg)]
                                } = 1 ;


* Count conforming values in each configuration

biggdiffcnt(pctchg,levchg) =
   SUM((sac,sacp)$nbiggdiff(sac,sacp,pctchg,levchg),1);

 DISPLAY
         " SAMCE after estimating", SAMCE,
         " SAMBALCHK after estimating", SAMBALCHK,
         SEM, VALDIFF, PERDIFF,
         SAMCENONZCNT, SAMCEMEAN, SAMCEMAX, SAMCEMIN, SAMCESTDEV,
         SAMCE1DEVLOCNT, SAMCE1DEVHICNT, SAMCE2DEVLOCNT, SAMCE2DEVHICNT,
         bigdiffv, bigdiffp, bigdiffvp,
         DENTROPY.L,
         chgintpct, chgintlev, nbiggdiff, biggdiffcnt
          ;

* Load and print the new estimated macro-sam, MACSAM_SOL, by aggregating the new
* solution micro-sam, SAMCE.

 MACSAM_SOL(ssn,ssnp) =
   SUM((sac,sacp)$(map_ss_sac(ssn,sac) and map_ss_sac(ssnp,sacp)),
        SAMCE(sac,sacp)) ;

*Computing and checking totals
 MACSAM_SOL("m_sam_tot",ssn) = SUM(ss2np, MACSAM_SOL(ss2np,ssn));
 MACSAM_SOL(ssn,"m_sam_tot") = SUM(ss2np, MACSAM_SOL(ssn,ss2np));
 macsam_bal_CHK(ssn) = MACSAM_SOL("m_sam_tot",ssn) - MACSAM_SOL(ssn,"m_sam_tot") ;

 display MACSAM_SOL, macsam_bal_chk ;

 display ss, aggr, aggr2, macset ;

 display "control macro-sam from excel file", MACSAM_EXOG ;
 display "macro-sam from proto-sam", MACSAM_ENDOG ;
 display "macro-sam from solution SAMCE", MACSAM_SOL ;

 display stderrs, stderrs_RC, stderrs_AGG, stderrs_MSAM, aggtotal0 ;
 display sig_RC_coef, sig_AGG_coef, sig_TV_coef, sig_MSAM_coef, sigm, alt ;

 coeff0(t_tot,sacn) = SUM(sacnp, coeff0(sacnp,sacn)) ;
 display coeff0 ;
 display SIGMAC, CMAC, SIG3MAC, SIG1MAC ;


*~~~~~~~~~~~~~~~~~~~ END OF PRELIMINARY REPORT ON RESULTS ~~~~~~~~~~~~~~~~~~~~