*~~~~~~~~~~~ SPECIFY TREATMENT OF CELLS ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

$ontext
STANDARD ASSUMPTIONS:
 IF opt_blocks and opt_macro and opt_RC_exog = 0, are:
    1. Cell constraints are all on values, IVAL(irow,icol)
    2. Constraints are set on all valid column and row sums, icol2 and irow2
    3. Prior errors are assumed to be additive
    4. Prior standard errors are set by SIGM setting 

Define column sums that can be constrained.
 Might want to exclude columns with only one entry. Or, alternatively,
 leave them in and define entry constraint in terms of coefficients.

 Can exclude any other columns for which column constraints do not
 make a lot of sense.
 
 Accounts which sum to zero by definition should probably be
 constrained exactly, with no error. Can be done by simply setting their
 prior standard error, sigmay_RC, to zero, which will happen automatically if
 the target column sum is zero. See section on defining sigmay_RC.
 

 FOR COEFFICIENTS
 By default, coefficient cells will have multiplicative errors.
 Natural to specify errors as multiplicative, since then the coefficients
 cannot become negative.

 The IF opt_blocks statement below, specifies values and multiplicative
 errors if cmac(ss,acmt) = 2
 
 However, this limits the adjustment domain in values, which may lead
 to odd results if the discrepancies in row and column targets are large.
 Additive errors widen the domain of adjustment, but may also lead to
 odd results.
 
OPT_BLOCKS alternative assumptions

  1. Choices determined for blocks of the SAM as specified in
     CMAC and SIGMAC

  2. Set accounts to be in terms of coefficients by turning off ivalue.
     ICOEFF is set below as NOT ivalue.

  3. Coefficients are treated "equally" and the estimation procedure will
     fail if there are large discrepancies in the accounts. Spreading
     large adjustments evenly across coefficients leads to odd results.
 
     In that case, it is better to work with values, which results
     in more assymmetric coefficient adjustment and large cells
     adjusting more.

   4. Set coefficients for cells in blocks of the SAM, defined in
      the macro-sam cmac(ss,ssp) using map_ss_sac(ss,sac).
      If cmac(ssn, ssnp) = 1, then the SAM cells in the corresponding
      block (defined by map_ss_sac) are all specified as coefficients.

   5. If cmac(ss,ssp) = 2, treat cells as values and errors as
      multiplicative.

   6. If standard errors are present in sigmac, use them.
 
 LARGE CELL VALUE CHANGES
 If cells have to change a lot, multiplicative errors may be
  infeasible so try additive errors, e.g.,.
      acell(irow,a)$(not izero(irow,a))          = yes ;
      acell(c,h)$(NOT izero(c,h))                = yes ;
      



Start with standard assumptions, then alter below

*~~~~~~~ For information ~~~~~~~~~~~~~~~

 icol2(sac)          column sums to be constrained with or without error
 acol(sac)           additive errors for column sum constraints
 row2(sac)           rows to be constrained
 ivalue(sac,sac)     cell VALUES to be estimated
 acell(sac,sac)      additive error for cells

$offtext

*Set all valid column constraints
 icol2(icol)                       = YES ;
 icol2(icol)$(countcol(icol) le 2) = NO ;
 acol(icol2)                       = YES ;

*Set row constraints. If row-col equality constraints are used, then
*this is turned off below, once sacbal is set.
 irow2(irow)                       = YES ;
 irow2(irow)$(countrow(irow) le 2) = NO ;

*Set all cells to values
 ivalue(irow,icol)$(not izero(irow,icol))   = YES ;

*Set additive errors for all value cells
 acell(irow,icol)$ivalue(irow,icol)         = YES ;

*End of standard assumptions

* ~~~~~~~~~~ opt_blocks alternative assumptions ~~~~~~~~~~

 Parameter
   testcmac(sac,sacp) For testing cmac specification ;

IF(opt_blocks = 1,

 testcmac(sacn,sacnp)   = 0 ;
 LOOP{(ssn,ssnp)$[cmac(ssn,ssnp) or sigmac(ssn,ssnp)],
  LOOP[(sacn,sacnp)$[map_ss_sac(ssn,sacn)   AND
                     map_ss_sac(ssnp,sacnp) AND
                    (not izero(sacn,sacnp))],
    IF{cmac(ssn,ssnp),
      testcmac(sacn,sacnp)  = testcmac(sacn,sacnp) + 1 ;

         acell(sacn,sacnp)                        = NO ;
         ivalue(sacn,sacnp)$(cmac(ssn,ssnp) = 1)  = NO ;
         ivalue(sacn,sacnp)$(cmac(ssn,ssnp) = 2)  = YES ;
      };
   sig3mac(sacn,sacnp)$sigmac(ssn,ssnp) = sigmac(ssn,ssnp) ;
*ends of loops
  ] ;
  } ;

 display "*#*#*# check specification of coefficient cells",
                testcmac, cmac, ivalue ;

*~~~~~ Blocks of Coefficients
$ontext
 The code above identifies cells to be treated as coefficients by
 reference to a macro-SAM
 Alternatively the blocks of coefficients can be hard coded.
$offtext

$INCLUDE 2_include\samest_coef_block.inc

*Government expenditure shares on commodities
* ivalue(c,g)$ivalue(c,g)                    = no ;

* If coefficients are tiny, specify cell as value rather than coefficient
* ivalue(irow,icol)$((not izero(irow,icol)) and (coeff0(irow,icol) lt 0.01))
*                                            = yes ;

*Set some defaults.
* If coefficients are negative, specify cell as value. Done again below.
 ivalue(irow,icol)$ineg(irow,icol)          = YES ;

*end if for opt_blocks
 ) ;

*End of opt_blocks alternative estimation assumptions

*In all circumstances, turn off irow2 if there is row-column equality
 irow2(sacbal)   = NO ;
 
*~~~~~~~~~~~~~~~~~~ END OF SPECIFY TREATMENT OF CELLS ~~~~~~~~~~~~~~~~~~~