*~~~~~~~~~~~~~~~~ ASSIGN PRIOR P_SAM ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

$onText
There are 3 procedures applied in this file.

    These procedures may result in SAM0 = P_SAM, IFF all adjustments
  have been made to P_SAM before it is loaded into the programme.
  NOTE:
    a.  these procedures may be redundant
    b.  BUT these procedures can be used to remove 'small' TVs
        from P_SAM0 as part of a process used to successively estimate
        a final SAM, e.g., increase the cutoff to get a first estimate,
        further condition the model and then reset the cutoff
        
        NOTE: A 'LARGE' VALUE FOR THE CUTOFF MAY RENDER THE PRIOR SAM
              INCOMPLETE. IDEALY THE CUTOFF SHOULD ZERO - all prior
              estimates of TVs should be based on 'evidence'. 
 
The procedures: 

  1.  Remove 'tiny' transaction values (TVs)
       a. Very small TVs may cause solution problems (even if they
          are relevant)
       b. The definiiton of 'tiny' is defind by the parameter
          'cutoff0' that is assigned in Excel (d_controls wksheet)
  2.  Identify Rows (irow) and Columns (icol) for inclusion the
      estimation and use irow and icol to assign sacbal.
       a. This MAY be a subset of P_SAM
  3.  Assign the SAM (SAM) used as the prior in the estimation   
       a. using irow and icol and sacbal
       b. compute differences between row and column sums
       c. compute some information on coefficients and non zeros
       
$offtext

*########### Remove tiny transacations using set SMCELL ##########

$ontext
 'SMALL' TVs in the prior P_SAM my cause problems when solving
  1. If the user does not want to remove small absolute values,
     then CUTOFF0 should be set at zero.
         Default value of cutoff is zero.

  2. If the user wants to remove small absolute values then CUTOFF0
     should be set at a low positive value, e.g. 0.005 (depends on
     units in P_SAM after scaling).
 
  3. Elements are in the set smcell if
     i.  they have a non-zero P_SAM cell; AND
     ii. the absolute value of this cell is less than cutoff  

$offtext

* ~~~~ Assign cutoff

 cutoff     = cutoff0 ;

Parameter
 cutoff_test      Test for negative or large cutoff
  ;
   
 cutoff_test$[(cutoff lt 0.0 ) OR (cutoff gt (10.0/samscal) )] = 1.0 ;
 
 ABORT$(cutoff_test GT 0.0)
 "Error cutoff0 out of range. Check cutoff in Excel", cutoff_test ;

Display cutoff0 ; 

 cutoff     = cutoff0 ;
 
*~~~~ Assign small cells set

 smcell(sacn,sacnp)$(P_SAM(sacn,sacnp) and (ABS(P_SAM(sacn,sacnp)) LT cutoff))
                                    = yes ;

*The cells corresponding to elements in smcell are set at zero.
 P_SAM(sacn,sacnp)$smcell(sacn,sacnp) = 0.0 ;

*Generate column and row sums
 P_SAM(t_tot,sacnp)    = sum(sacn,  P_SAM(sacn,sacnp)) ;
 P_SAM(sacn,t_tot)     = sum(sacnp, P_SAM(sacn,sacnp)) ;

$ontext
 Warns if removing cells results in sparse rows or columns (defined as
 5 nonzero entries remaining). This could indicate that important information
 is being lost and could cause potentially serious problems with the
 estimation process.

 Interpretation of nonzero cells in sparsewarn:

  1 - row is sparse
  2 - column is sparse
  3 - both column and row are sparse

$offtext

 sparsewarn(sacn,sacnp)
                    = 1$(smcell(sacn,sacnp) AND SUM((sacnpp)$
                                                 P_SAM(sacn,sacnpp),1) < 5)
                      + 2$(smcell(sacn,sacnp) AND SUM((sacnpp)$
                                                 P_SAM(sacnpp,sacnp),1) < 5)  ;
                                                 
* Small cells and near-small cells counted

 smlcellcount     =  SUM(smcell,1) ;

 nearcutoffcount  =  SUM[(sacn,sacnp)
                      ${[ABS(P_SAM(sacn,sacnp))>cutoff] AND [ABS(P_SAM(sacn,sacnp))<(10*cutoff)]
                             } , 1.0 ] ;

*~~~~~~~~~ Defining the ROWS and COLUMNS included in the P_SAM ~~~~~~~

*~~~~~ Assign sets icol irow (Non zero columns and rows) and sacbal
$ontext
 Assignment steps
   1.  Count nonzero elements in columns and rows
   
   2.  Assign sets icol and irow
         a.  Define column and row sets for all accounts with any nonzero
             elements in the corresponding row and column.
         b.  A P_SAM account might have a row or column with all zeros;
             those accounts can be excluded from an estimation procedure,
             IFF the paired row/column does not have entries that sum to ZERO.
         
   3.  Assign sacbal for common accounts in icol and irow
         a.  Default (default_SAM = 1) assumes common rows and columns
             are to be estimated, which is standard for estimating SAMs.
         b.  By default, sacbal includes common set elements in irow and icol.
         c.  If default_SAM is 0 then sacbal is empty and no row-column sum
             constraints are imposed.
             
   4.  Assign Values for SAM estimating using irow and icol
   
   5.  Define columns and rows with nonzero sums.
         a.  Nonzero is defined as significant absolute value so that
             column coefficients can be computed safely.

 NOTES:
   A.  sacbal need not include the whole matrix, and the code will work for
         rectangular matrices such as input-output tables rather than square
         SAMs. In this case, corresponding rows and columns
         may be equal, OR may NOT be equal.
         
   B.  If a user wants to estmate a subset of the P_SAM, then icol and irow would
         need to be redefined to include only those accounts.
           The code has not been tested for estimating sub-matrices, but
           it should be feasible to do do. One must also check sacbal to
           ensure that row and column sum equality is imposed only for those
           accounts for which it is valid.

   C.  If estimating only part of a P_SAM, sacbal needs to be only set to
         common rows and columns that include all elements and hence
         should equate.

   D.  For default, count nonzero elements in columns and rows and
         set icol and irow to include all columns and rows with
         any nonzero elements.
         
$offtext

*~~ Count nonzero elements in columns and rows

 countcol(sacn)    = SUM[sacnp$P_SAM(sacnp,sacn),1] ;
 countrow(sacn)    = SUM[sacnp$P_SAM(sacn,sacnp),1] ;

*~~ Assign sets icol and irow

 icol(sacn)$countcol(sacn)   = yes ;
 irow(sacn)$countrow(sacn)   = yes ;

*~~ Assign sacbal for common accounts in icol and irow

 sacbal(sacn)$(icol(sacn) and irow(sacn) and default_SAM) = yes ;

option countcol:0, countrow:0 ;

*~~~~~~~~ Assign Values for SAM Estimation ~~~~~~

*~~ Initial P_SAM for estimation. It may be a subset of whole P_SAM

 P_SAM0(irow,icol)     = P_SAM(irow,icol) ;

 P_SAM0(t_tot,icol)  = sum[irow, P_SAM0(irow,icol)] ;
 P_SAM0(irow,t_tot)  = sum[icol, P_SAM0(irow,icol)] ;

 p_colsum(icol)      = P_SAM0(t_tot,icol) ;
 p_rowsum(irow)      = P_SAM0(irow,t_tot) ;
                     
 SAMBALCHK(sacbal)   = P_SAM0(t_tot,sacbal) - P_SAM0(sacbal,t_tot) ;

*~~ Identify percentage differences between row and column totals

 SAMBALCHKP(sacbal)$P_SAM0(sacbal,t_tot)     =
    100*[P_SAM0(t_tot,sacbal) - P_SAM0(sacbal,t_tot)]/P_SAM0(sacbal,t_tot) ;

*~~ Define columns and rows with nonzero sums.

 icolnz(icol)${ABS[P_SAM0(t_tot,icol) gt 0.01]} = yes ;

* compute column coefficients.

 coeff0(irow,icolnz)      = P_SAM0(irow,icolnz)/P_SAM0(t_tot,icolnz) ;

* Find negative cell entries
 izero(irow,icol)$(P_SAM0(irow,icol) eq 0.0 )   = yes ;
 nonzero(irow,icol)$(not izero(irow,icol))      = yes ;
 
 ineg(irow,icol)${[P_SAM0(irow,icol) lt 0.0 ] and nonzero(irow,icol)} = yes ;
 
*~~~~~~~~~~~~~~~~ END OF ASSIGN PRIOR P_SAM ~~~~~~~~~~~~~~~~~~~~~~~~~~