$TITLE SAM estimation program. Version 5. Feb 2023
$OFFSYMLIST OFFSYMXREF OFFUPPER
$ONEMPTY

*################ GENERAL INFORMATION ####################################
$ontext
This programme was developed by Sherman Robinson (IFPRI)
                           with Scott McDonald   (cgemod)
$offtext

*################# 1. PROGRAMME LAYOUT #######################################
$ontext
The section and subsection headings are listed below.
     At the end of each section there is an 'END OF' section statement.
     At the end of the programme there is a 'THE END' statement.
     At the end of each IF statemet there is an 'END if' statement.
     At the end of each LOOP statemet there is an 'END loop' statement.
     
* ------ SCHEMATIC OUTLINE OF PROGRAMME
The model code consists of a number of files; the relationships between
these files is illustrated below.

 samest.gms                                MASTER programme file
     |
     |  2. SETGLOBAL ASSIGNMENTS         Identify source data input and output files
     |
     |  3. SET DECLARATIONS              Declare all sets used in the programme
     |
     |  4. PARAMETER DECLARATIONS        Declare all parameters used in the programme 
     |   
     |  5. DATA ENTRY                    All sets, data and controls are loaded here
     |
     |<----- samest_dataload.inc         Load model sets, data & controls
     |
     |6. DESCRIPTIVE STATISTICS          Descriptive statistics based on prior data
     |
     |<----- samest_stats.inc
     |
     |7. ASSIGNING SETS AND PARAMETERS FROM DATA 
     |
           | 7a. ASSIGN PRIOR SAM
           |<----- samest_prior.inc
           | 
           | 7b ASSIGN PRIOR MACRO SAM
           |<----- samest_msam_endog.inc 
           | 
           | 7c Assign Values to AGGAGG and AGGTOTAL0
           |<----- samest_aggtotal.inc
           | 
           | 7d. SPECIFY TREATMENT OF CELLS
           |<----- samest_cells.inc
           | 
           | 7e. Define column and row target values
           |<----- samest_RC_targets.inc
           | 
           | 7f. FORCE CONSISTENCY FOR CONSTRAINT & ERROR SPECIFICATIONS
           |<----- samest_consist.inc
     |
     |8. SET ERROR BOUNDS AND SUPPORT SETS
     |
     |<----- samest_err_supp.inc
     |
     |9. DECLARING VARIABLES 
     |
     |10. INITIALISE VARIABLES AND SET BOUNDS
     |
     |<----- samest_varinit.inc
     |
     |13. DECLARING EQUATIONS 
     |
     |14. ASSIGNING EQUATIONS 
     |
     |15. DEFINE MODEL 
     |
     |16. SOLVE MODEL 
     |
     |17. PRELIMINARY REPORT ON RESULTS
     |
     |<----- samest_anal.inc
     |
     |18. EXPORTING RESULTS
 
$offtext    
*################# END OF PROGRAMME LAYOUT ###############################
 
*################# 2. SETGLOBAL ASSIGNMENTS #######################################

*~~~ Assign name of Excel workbook that contains data for use in SAMEST
$ontext
* NOTES:
      1.  The data for estimation are from Excel, e.g., xl_data.xlsx
              the format is xlsx - exclude '.xlsx' from SETGLOBAL
          The programme converts the data in Excel to GDX format
              Assign 'xl_data.xlsx' to 'samest_data' by SETGLOBAL below
      2.  The Prior SAM can be in
          a.  the Excel workbook
                 Assign 'data_in' to 'p_sam_in' by SETGLOBAL below
          b.  or in a separate GDX file, e.g., - 'gdx_data.gdx'
                 Assign 'gdx_data' to 'p_sam_in' by SETGLOBAL below
      3.  The Prior macro-SAM can be in
          a.  the Excel workbook
                 Assign 'data_in' to 'amcsam_in' by SETGLOBAL below
          b.  or in a separate GDX file, e.g., - 'gdx_data.gdx'
                 Assign 'gdx_data' to 'macsam_in' by SETGLOBAL below
      3.  Assign a label, e.g., 'data_out', that IDs two project specific
          outputs - Statistics relating to the priors (dataout_prior-stats.gdx
          from samest_stats.inc)and the estimated SAM and related statistics
          (data_out_sam_out.gdx from samest_5.gms)
              Assign 'data_out' to 'dstats' by SETGLOBAL below

$offtext

*~~~~~~ Substitution to control Dataload
*~~ Identify the Excel workbook
$SETGLOBAL samest_data   country_BTN_sm
*$SETGLOBAL samest_data   country_BTN_mid
*$SETGLOBAL samest_data   country_BTN_cgemod1

*~~ Identify the location of the Prior SAM (data_in if the prior is in Excel) else a GDX file
$SETGLOBAL p_sam_in      data_in

*~~ Identify the location of the EXOGENOUS Macro SAM (data_in if the prior is in Excel) or GDX file
$SETGLOBAL macsam_in     data_in

*~~~~~~ Substitution to Assign labels for project specific outputs

$SETGLOBAL dstats        country_BTN_sm
*$SETGLOBAL dstats        country_BTN_mid
*$SETGLOBAL dstats        country_BTN_cgemod1
 
*################# 3. SET DECLARATIONS #######################################

SETS

* SAM Sets are assigned in Excel worksheet sets

*~~~~~ Micro SAM Sets from Excel

 sac                 SAM accounts
 c(sac)              Commodities
 m(sac)              Margins
 a(sac)              Activities
 f(sac)              Factors
 l(sac)              Labour Factors
 k(sac)              Capital Factors
 lnd(sac)            land factors
 h(sac)              Households
 g(sac)              Government
 gtind(sac)          Indirect tax accounts
 gtdir(sac)          Direct tax accounts
 e(sac)              Enterprises
 i(sac)              Investment
 w(sac)              Rest of the world
 

 emrgn(sac)          export margin account
 etax(sac)           export tax account 

*~~~ Dynamic Micro SAM Sets set in programme

 sacbal(sac)         accounts with row-column balance
 sacn(sac)           all accounts except totals

*~~~~~ MAcro SAM Sets from Excel
 ss                  MACSAM_ENDOG macro-sam accounts
 
*~~~ Dynamic MAcro SAM Sets set in programme
 
 ssn(ss)             ss except for m_sam_tot
 ss2n(ss)            ss except for m_sam_tot and fctr3
 ss3n(ss)            all ss except m_sam_tot and other3

*~~~~~ Sets for Row and Column Constraints

 rwcl                row and column constraints  /row1, col1/
 rwcl1               row and column data         /stderr_RC, rowsum1, colsum1 /
                  
*~~~~~ Sets for Errors and Error weights
                   
 jwt                 potential weights for error support set        /1*7/
 jwt_RC(jwt)         actual weights for error support set for Row and Column
 jwt_AGG(jwt)        actual weights for error support set for specified aggregates
 jwt_TV(jwt)         actual weights for error support set for specified Transaction Values
 jwt_MSAM(jwt)       actual weights for error support set for specified MACRO SAM estimates
                  
 esuppset            Names of error support set control parameters
                  
 sig                 set of sigmays for rows macro and cells
*                                               /sig1, sig2, sig3 /
 alt                 set of alternative prior standard errors
*                                            /alt0, alt1, alt2 /

* Sets assigned within the programme

* aggagg(sac,sac,agg) Macro aggregator matrix

*~~~~~ Sets for identifying cells with specific characteristics

 smcell(sac,sac)     SAM cells with abs value < cutoff are removed
 ineg(sac,sac)       cells with negative values in data
 izero(sac,sac)      cells with zero entry
 nonzero(sac,sac)    cells with nonzero entry
 
*~~~~~ Sets for identifying cells to be estimated or fixed

 icoeff(sac,sac)     cell COEFFICIENTS to be estimated
 ivalue(sac,sac)     cell VALUES to be estimated
 estimate(sac,sac)   cells to be estimated and not fixed
 ifixv(sac,sac)      cell value fixed with no error
 
*~~~~~ Sets to define 'nature' of errors by cells
                     
 acell(sac,sac)      additive error for cells
 lcell(sac,sac)      logarithmic error for cells
                     
*~~~~~ Sets to defined rows and columns in estimation

 icol(sac)           columns included in estimation
 icol2(sac)          column sums to be constrained with or without error
 icolnz(sac)         columns with nonzero sum
 irow(sac)           rows included in estimation
 irow2(sac)          rows to be constrained
 acol(sac)           additive errors for column sum constraints

*~~~~~ Sets for error checks on constraints

 icheck(sac,sac)     Error check for cell constraints
 icheck2(sac)        Check set for irow or icol but not in sacn
 
*~~~~~ Sets for Aggregate Control Sets that are assigned in Excel

 aggr                potential aggregate control totals
 aggr2(aggr)         aggregate constraints actually imposed

 map_ss_sac(ss,sac)  mapping from sac to ss for aggregating sam to macsam
 macset(ss,ss)       mac-sam constraints to be imposed
 aggt                column names for AGGMAT
                   
*~~~~~ Set of SAMOPT_TV_TV(sac,sacp,nsam) names

 nsam               Names of control elements for SAMOPT_TV
 
*~~~~~ Set for controlling model assignments from Excel

 d_cons             set of controls for data assignment
 m_cons             set of controls for model configuration
 
 ;

ALIAS (sac,sacp,sacpp,sacppp), (sacn,sacnp,sacnpp,sacnppp),
      (sacbal,sacbalp),
      (ssn,ssnp), (ss,ssp), (ss2n, ss2np),
      (c,cp), (m,mp), (a,ap),
      (f,fp), (l,lp), (k,kp),
      (h,hp), (e,ep),
      (g,gp), (gtind,gtindp), (gtdir,gtdirp),
      (i,ip),
      (w,wp)
   ;
      
*################# END OF SET DECLARATIONS ##############################

*##################### 4. PARAMETER DECLARATIONS ############################

PARAMETER

*########## Parameters used for data entry ###############

*~~~~~~~~~ Parameters for Prior SAM
 P_SAM(sac,sac)        Prior SAM for this model
 P_SAM0(sac,sac)       orignial best prior SAM after scaling
 
*~~~~~~~~~ CONTROL TOTALS

*~~~ Parameters for Macro-SAM Controls
 MACSAM0(ss,ss)        macro-sam used for control totals in estimation
 macsam_exog(ss,ss)    macro-sam read in excel file
 macsam_endog(ss,ss)   macro-sam calculted from proto-sam

*~~~ Parameters for Row and Column Control Totals

 coltarget(sac)         target column sum
 rowtarget(sac)         target row sum
 
*~~~ Parameters for Aggregate Control Totals
                   
 aggtotal(aggr)        Aggregate control totals used in estimation
 AGGTOTAL0(aggr)       Aggregate totals from excel input file
 AGGTOTAL2(aggr)       Aggregate totals from micro proto-sam SAM
 AGGMAT(aggr,aggt)     Aggregate totals and prior standard errors from SAM

 macsam_sol(ss,ss)     macro-sam aggregated from solution samce
 
 macsam_bal_chk(ss)    macro-sam row and column balance check
 
 sigmac(ss,ss)         standard errors for blocks of sam
 cmac(ss,ss)           specification of coefficients for blocks of sam
 sig3mac(sac,sac)      sig3 for blocks of sam cells specified in cmac and sigmac
 sig1mac(sac)          sig1 for blocks of sam columns or rows in cmac and sigmac
 macmnz(ss,ss)         non-zero entries of macsam

* macro aggregator
 aggagg(sac,sac,aggr)  AGGREGATE controls aggregator matrix

*~~~~~ Parameters Used to scale Prior SAM and Aggregates

 samscal               SAM scaling factor
 samautoscal           SAM autoscaling option
 samscal0              Initial SAM scaling factor   
 samscaltarg           SAM autoscaling target       
 samscalprop           SAM autoscaling proportion
 samscalman            Manual SAM scaling factor
 macscale              Scale choice for MACSAM_EXOG
 
*~~~~~ Parameters for Standard errors
                    
 stderrs(sig,alt)      default standard errors
 stderrs_RC(sac)       non-default standard errors for column and row totals
 stderrs_AGG(aggr)     non-default standard errors for aggregate control totals
 stderrs_MSAM(ss,ss)   non-default standard errors for macro-sam control totals

 rowcol_tot(sac,rwcl1) row and column sum controls and stderrs1

*############## Parameters used to control model flow ###################

 epsilon               epsilon value for cross-entropy minimand   /1e-6/

 d_controls(d_cons,*)  initial values for data controlling parameters
 m_controls(m_cons,*)  initial values for parameters to control model
* err_values
 
*~~~~~ Parameters to select controls implemented
                     
 default_SAM           if 1 Assume that sacbal row-sum balance is imposed  /1/

*~~~~~ Options to control settings
 opt_blocks            if 1 Treat blocks of SAM specially
 opt_agg               if 1 Use aggregate control totals
 opt_RC_exog           if 1 Use special row-column constraints
 opt_tv                if 1 Use special transaction values for individual SAM elements
 opt_msam              if 1 Use standard macro-sam control totals

*~~~~~ Parameters for identifying cells with specific characteristics
                   
 cutoff0                value below which small absolute values are set to zero
 smlcellcount           Number of small cells
 nearcutoffcount        Number of values between cutoff and 10 times cutoff value
 sparsewarn(sac,sacp)   Cutoff cells in sparse rows or columns
                      
 countcol(sac)          number of nonzero cells in columns
 countrow(sac)          number of nonzero cells in rows
                      
 cutoff                 lower bound on absolute cell values
                      
 target                 specification of row and column totals
                      
 esupp(esuppset)        number of elements in error support sets for blocks of SAM
 sigm                   select the vector of std errors on errors imposed
 
*######## Parameters used in estimation procedure ##################

* tsam0(sac,sacp)        initial sam
 coeff0(sac,sacp)       column coefficients

 p_colsum(sac)          prior column sums for P_SAM0
 p_rowsum(sac)          prior row sums for P_SAM0
 
 colsum0(sac)           initial column sums for estimation
 rowsum0(sac)           initial row sums for estimation
 
 sambalchk(sac)         col sums minus row sums for sacbal
 sambalchkp(sac)        sambalchk as percent of column sum                 
            
 SAMOPT_TV(sac,sac,nsam) Control elements for individual SAM entries
 
*~~~~~ Parameters for Errors and Error weights

 vbar_RC(sac,rwcl,jwt)  Error support set 1 for row and column constraints
 vbar_AGG(aggr,jwt)     Error support set 2 for aggregate constraints
 vbar_TV(sac,sacp,jwt)  Error support set 3 for cell constraints
 vbar_MSAM(ss,ss,jwt)   Error support set 4 for macro-sam constraints

 wbar_RC(sac,rwcl,jwt)  Weights on error support set RC
 wbar_AGG(aggr,jwt)     Weights on error support set MAC
 wbar_TV(sac,sacp,jwt)  Weights on error support set TV
 wbar_MSAM(ss,ss,jwt)   Weights on error support set MSAM

 sigm_RC                Default std error parameter for sigmay_RC
 sigm_AGG               Default std error parameter for sigmay_AGG 4 and 5
 sigm_TV                Default std error parameter for sigmay_TV
*sigm4                  Default std error parameter for sigmay_MSAM
                      
 sig_RC_coef(sac,rwcl)  Prior coefficient used to calculate sigmay_RC
 sig_AGG_coef(aggr)       Prior coefficient used to calculate sigmay_AGG
 sig_TV_coef(sac,sacp)  Prior coefficient used to calculate sigmay_TV
 sig_MSAM_coef(ss,ss)   Prior coefficient used to calculate sigmay_MSAM

 sigmay_RC(sac,rwcl)    Prior standard error for column sums
 sigmay_AGG(aggr)       Prior standard error for macro aggregates
 sigmay_TV(sac,sacp)    Prior standard error for cell constraints
 sigmay_MSAM(ss,ss)     Prior standard error for macro-sam constraints

*########## Parameters used for descriptive statistics ##############

 SAMNONZCNT             Number of nonzero elements in SAM
 SAMMEAN                Mean of nonzero elements in SAM
 SAMMAX                 MAX of elements in SAM
 SAMMIN                 MIN of elements in SAM
 SAMSTDEV               Standard deviation of nonzero elements in SAM
 SAM1DEVLOCNT           No of nonzero elements less than mean - 1 stddev in SAM
 SAM1DEVHICNT           No of nonzero elements greater than mean + 1 stddev in SAM
 SAM2DEVLOCNT           No of nonzero elements less than mean - 2 stddev in SAM
 SAM2DEVHICNT           No of nonzero elements greater than mean + 2 stddev in SAM
     ;

*##################### END OF PARAMETER DECLARATIONS ####################

*##################### 5. DATA ENTRY ########################################

*~~~~~ The Sets and Data are read in from GDX
$ontext
 Sources for Sets and Data are identified by SETGLOBAL 'varname'
    a.  Sets come from Excel - samest_data
    b.  Prior SAM data can come from Excel of directly from GDX
        i.  If prior in Excel - p_sam_in is data_in
        ii. If prior in GDX - p_sam_in is 'gdxname'
    c.  Other Data come from Excel - samest_data

 i.   Coverts sets and data in Excel to GDX    
 ii.  Load sets and data 
 iii. Define sets that exclude total and other elements
 iv.  Set DEFAULTS to control the model
 v.   Assign parameters for Aggregate Targets
 vi.  Select ERROR Support Sets
 vii  SAM scaling
   
$offtext

$INCLUDE 1_data\samest_dataload.inc

*################## END OF DATA ENTRY #################################

*##################### 6. DESCRIPTIVE STATISTICS ###########

*~~~~~ Descriptive statistics for the Prior SAM
$ontext
  A series of descriptive statistics for the Prior SAM are
  computed.
   They are exported as %****%_prior-stats.gdx
   The label **** is assigned using SETGLOBAL
   
  IF GAMS is run with GDX creation these results ?????

$offtext

$INCLUDE 2_include\samest_stats.inc

*##################### END of DESCRIPTIVE STATISTICS ###########

*##################### 7. ASSIGNING SETS AND PARAMETERS FROM DATA ########

*~~~~~~~~~ 7a. ASSIGN PRIOR SAM ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

$ontext
 This file 
      1.  Eliminates small cell values
      2.  Checks for a sparse matrix
      3.  assigns sets that control the prior SAM for estimation
             included row (irow) and column (icol) accounts
             sacbal for common row and columns included
      4.  assigns prior SAM values (scaled in dataload.inc)
$offtext

$INCLUDE 2_include\samest_prior.inc

*~~~~~~~~~ END OF ASSIGN PRIOR SAM

*~~~~~~~~~ 7b. ASSIGN PRIOR MACRO SAM ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

$ontext
 This section produces an aggregated macro-SAM, MACSAM_ENDOG, from the
 prior SAM, P_SAM. This macro-SAM may be used to assign a macro SAM
 used to provide control totals for the estimation process,
 IFF an ‘ideal’ exogenously defined macro-SAM has NOT been provided
 in the Excel database.
 
 This section ONLY provides the Transaction Values for the macro SAM.
 The ERROR BOUNDS AND SUPPORT SETS are defined in samest_err_supp.inc
 below.
 
 The choice about an EXOGENOUS (ideal) or ENDOGENOUS(alternative) is
 made in the 'controls' worksheet in the Excel workbook.
 
 IDEAL:
      1.  An exogenous macro SAM that is
          a. complete and consistent
          b. distinguishes between the factors
             i.    labour
             ii.   capital
             iii.  land
          c. distinguishes between indirect and direct taxes
          d. includes domestic margins
      2.  If the exogenous macro SAM is INcomplete and/or
          INconsistent
          a. limit the cells in the macro SAM used as controls
          b. place wider bounds on controls used

 ALTERNATIVES:
      1.  Do not impose macro SAM controls
      2.  Derive an implicit macro SAM, 'macsam_endog' from the prior
          SAM, P_SAM
          a. The endogenous macro SAM will be INcomplete and/or
          b. The endogenous macro SAM will be INconsistent
              THEREFFORE
              i.  limit the cells in the macro SAM used as controls
              ii. place wide bounds on controls used
      
 The aggregation is controlled by the set map_ss_sac, which is defined
 from subsets in the Excel file. Sets are defined in the main program
 and loaded fom Excel.
$offtext

$include 2_include\samest_msam_endog.inc

IF{[m_controls("con_macsam","base") = 0.0],

 MACSAM0(ss,ssp)                        = macsam_exog(ss,ssp) ;
 
  } ;
 
IF{[m_controls("con_macsam","base") = 1.0],

 MACSAM0(ss,ssp)$(MACSAM0(ss,ssp) eq 0) = macsam_endog(ss,ssp) ;
 
  } ;

$ontext  
Define macro-sam constraints actually imposed, macset(ss,ss). Initially
  read from excel file (stderrs4), but correct it here if some macsam
  elements are always zero.

$offtext
*~~~~ Control for non zero elements in P_SAM and MACSAM0
* In MACSAM0
 macmnz(ss,ssp)$MACSAM0(ss,ssp)         = YES ;

* In P_SAM
 macmnz(ssn,ssnp) = SUM{(sac,sacp)
                        $[map_ss_sac(ssn,sac)
                          and map_ss_sac(ssnp,sacp)
                          and P_SAM(sac,sacp)], 1} ;

* Add in total row and column if they have non-zero elements
 macmnz("m_sam_tot",ssn) = SUM[ss2np$MACSAM0(ss2np,ssn), 1 ] ;
 macmnz(ssn,"m_sam_tot") = SUM[ss2np$MACSAM0(ssn,ss2np), 1 ] ;

 macset(ss,ssp)$stderrs_MSAM(ss,ssp)    = YES ;

 macset(ss,ssp)$(macmnz(ss,ssp) eq 0)   = NO ;

* Do not use macro-sam control totals if opt_msam NE 0.0
 macset(ss,ssp)$(opt_msam NE 0.0)       = NO ;

 display macset, macmnz ;

*~~~~~~~~~ END OF ASSIGN PRIOR MACRO SAM 

*~~~~~~~~~ 7c Assign Values to AGGAGG and AGGTOTAL0 ~~~~~~~~~~~~~~~~~~~~~~

$ontext

   Use AGGREGATE constraints if opt_AGG = 1.0.

   AGGAGG(sac,sacp,aggr) is a mapping of micro sam entries to target AGGREGATE totals
 in AGGTOTAL0. A number of AGGREGATE control totals are pre-specified in
 samest_aggtotal.inc, and the set is specified in the excel file, sheet "aggr".

   The controlling set on AGGTOTAL0 is aggr, i.e., all the AGGREGATE control
 totals that might POTENTIALLY be used in the programme. Those ACTUALLY
 imposed are a subset - aggr2 - of aggr.

$offtext

* Assign members of aggr2 based on AGGMAT

*f aggr2(aggr)$AGGMAT(aggr,"value_agg")  = YES ;

$INCLUDE 2_include\samest_aggtotal.inc

*~~~~~~~~~ END OF Assign Values to AGGAGG and AGGTOTAL0 

*~~~~~~~~~ 7d. SPECIFY TREATMENT OF CELLS ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

$ontext

  This stage has 2 include files - one within samest_cells.inc

  SAMEST_COEF_BLOCK.INC

    The code in samest_cells.inc identifies cells to be treated as
 coefficients by reference to a macro-SAM.
 
    This include file illustrates hard coding of coefficients.

  SAMEST_CELLS.INC
    This file identifies cells to be
  i.    included in the estimates
  ii.   cells to be estimated as values
  iii.  sets cells estimated as values to have additive error
  iv.   cells to be estimated as coefficients by references to
        macro SAM

$offtext

$INCLUDE 2_include\samest_cells.inc

*~~~~~~~~~ END OF SPECIFY TREATMENT OF CELLS 

*~~~~~~~~~ 7e. Define column and row target values ~~~~~~~~~~~~~~~~~~~~~~~

$ontext
The row and column targets are defined in this file. 
 1. Check the differences between the totals of the rows and columns     
 2. Assign row and column total exogenously from worksheet
      If opt_RC_exog
 3. Assign targets based on Row and column controls (rowsum0 and colsum0)
      Target = 1 - target totals are mean of prior SAM row & column totals
      Target = 2 - target totals are prior SAM column totals
      Target = 3 - target totals are prior SAM row totals 
 4. Assign Targets for blocks specified as coefficients
      This requires DEFAULT1 and CMAC.
 5. Assign Targets for blocks specified as coefficients
      IF opt_tv = 1

$offtext

$INCLUDE 2_include\samest_RC_targets.inc

*~~~~~~~~~ END OF DEFINE COLUMN AND ROW TARGET VALUES 

*~~~~~~~~~ 7f. FORCE CONSISTENCY FOR CONSTRAINT & ERROR SPECIFICATIONS ~~~

$ontext
This file checks for consistency in the specifications for cells
 1.  Ensure cells NOT specified as values (ivalue) are specified as
     coefficients.    
  2. Check for errors in setting of constraints.

$offtext

$INCLUDE 2_include\samest_consist.inc


*~~~~~~~~~ END OF FORCE CONSISTENCY FOR CONSTRAINT & ERROR SPECIFICATIONS
   
*##################### END OF ASSIGNING SETS AND PARAMETERS FROM DATA ###

*##################### 8. SET ERROR BOUNDS AND SUPPORT SETS ################

$ontext
    This include file sets up the error support sets. The calibration is
  controlled from EXCEL.
   
    The operation depends upon the number of elements in the error support
  set, 'errsupp'. This can vary for each of the errors. These are set in the
  worksheet 'err_values' for each element
     IF Esupp_* = 7  - prior is assumed uninformative, i.e., UNIFORM
     IF Esupp_* = 3  - prior is assumed informative, with a two moment
                       posterior distribution (mean, variance)
     IF Esupp_* = 5  - prior is assumed informative, with a four moment
                       posterior distribution (mean, variance, skewness, and kurtosis). 
     
$offtext

$INCLUDE 2_include\samest_err_supp.inc

*##################### END OF SET ERROR BOUNDS AND SUPPORT SETS #########

*##################### 9. DECLARING VARIABLES ###############################

VARIABLES

*~~~ TVs and Coefficients
 SAM(sac,sacp)           SAM values
 COEFF(sac,sacp)         SAM coefficients
 COLSUM(sac)             column sums
 ROWSUM(sac)             row sums
                       
*~~~ Macro SAM         
 MACSAM(ss,ssp)          macro-sam
 
*~~~ Errors and Error weights
 ERR_RC(sac,rwcl)        Error value on Row and Column sums
 ERR_AGG(aggr)           Error value for aggregate control totals
 ERR_TV(sac,sacp)        Error value for TV constraint
 ERR_MSAM(ss,ss)         Error value for macro-MSAM constraints

 W_RC(sac,rwcl,jwt)      Error weights on Row and Column sums       
 W_AGG(aggr,jwt)         Error weights for aggregate control totals         
 W_TV(sac,sacp,jwt)      Error weights for TV constraint            
 W_MSAM(ss,ss,jwt)       Error weights for macro-MSAM constraints   

*~~~ Objective function
 DENTROPY                Entropy difference (objective)

 ;

*##################### END OF VARIABLE DECLARATIONS #####################

*##################### 10. INITIALISE VARIABLES AND SET BOUNDS ##########

$onText

This file
  1.  Initializes variables
  2.  Define bounds for cell values
  
This file has 2 parts each with several steps
  1.  Initialize variables
       a. TVs and Coefficients
       b. Macro SAM
       c. Errors and Error weights
       d. Objective function
       
  2.  Define bounds on cell values
       a. Set bounds and fix variables that need to be fixed
       b. Set lower bound of zero on coeffs
       c. Fix coeff to zero if it is in izero
       d. Upper and lower bounds on the error weights
       e. Fix ZERO elements of aggregate-sag AGGSAM  and macro-sam MACSAM

$offText

$INCLUDE 2_include\samest_varinit.inc

*##################### END OF DEFINE BOUNDS FOR CELL VALUES #############

*##################### 13. DECLARING EQUATIONS ##############################

EQUATIONS

*~~~ TVs and Coefficients
 SAMFLOW1(sac,sac)     SAM flows with additive errors
 SAMFLOW2(sac,sac)     SAM flows with logarithmic errors
 SAMCOEFF1(sac,sac)    SAM coefficients with additive errors
 SAMCOEFF2(sac,sac)    SAM coefficients with logarithmic errors
 COLSUMEQ1(sac)        column sums with additive errors
 ROWSUMEQ1(sac)        row sums with additive errors

 COLSUMDEF(sac)        define column sums
 ROWSUMDEF(sac)        define row sums
 COEFFDEF(sac,sac)     define coefficients

 SAMEQ(sac)            row and column sum constraint

*~~~ Macro constraints
 MACSAMEQ(ss,ss)       macro-sam definition
 MACSUMEQ1(ss)         column sums of macro-sam
 MACSUMEQ2(ss)         row sums of macro-sam
 MACSAMEQ2(ss,ss)      macro-sam constraints

 AGGREQ(aggr)           Aggregate control total constraints

*CELLVAL(sac,sacp)   Cell Value constraints

 EXPORTEQ2(sac)        Domestic production greater than exports

*~~~ Errors and Error weights
 ERROR_RC_EQ(sac)      definition of column error term for Row Col errors       
 ERROR_R_RC_EQ(sac)    definition of row error term for Row Col errors         
 ERROR_AGG_EQ(aggr)    definition of error for aggregate contorls          
 ERROR_TV_EQ(sac,sac)  definition of error for transaction values     
 ERROR_MSAM_EQ(ss,ss)  definition of error for Macro SAM              

 SUMW_RC(sac,rwcl)     Sum of weights for Row Col errors
 SUMW_AGG(aggr)        Sum of weights for aggregate controls
 SUMW_TV(sac,sac)      Sum of weights for transaction values
 SUMW_MSAM(ss,ss)      Sum of weights for Macro SAM
 
*~~~ Objective function
 ENTROPY               Entropy difference definition

 ;
*##################### END OF EQUATION DECLARATIONS #####################

*##################### 14. ASSIGNING EQUATIONS ##############################

*~~~~~~~~ SAM value and coefficient constraints
 SAMFLOW1(irow,icol)$(ivalue(irow,icol) and acell(irow,icol))..
  SAM(irow,icol)   =E= P_sam0(irow,icol)  + ERR_TV(irow,icol) ;

 SAMFLOW2(irow,icol)$(ivalue(irow,icol) and lcell(irow,icol))..
  SAM(irow,icol)   =E= P_sam0(irow,icol)*EXP(ERR_TV(irow,icol)) ;

 SAMCOEFF1(irow,icol)$(icoeff(irow,icol) and acell(irow,icol))..
  COEFF(irow,icol)*coltarget(icol)  =E=
  (coeff0(irow,icol) + ERR_TV(irow,icol))*coltarget(icol) ;

 SAMCOEFF2(irow,icol)$(icoeff(irow,icol) and lcell(irow,icol))..
  COEFF(irow,icol)*coltarget(icol)  =E=
  coeff0(irow,icol)*EXP(ERR_TV(irow,icol))*coltarget(icol) ;

*~~~~~~~~ Column and row sum constraints
 COLSUMEQ1(icol2)$(acol(icol2))..
  COLSUM(icol2)     =E= coltarget(icol2) + ERR_RC(icol2,"col1") ;

 ROWSUMEQ1(irow2)..
  ROWSUM(irow2)     =E= rowtarget(irow2) + ERR_RC(irow2,"row1") ;

*~~~~~~~~~ Define column and row sums and coefficients
 COLSUMDEF(icol)..
  COLSUM(icol)      =E= SUM(irow, SAM(irow,icol)) ;

 ROWSUMDEF(irow)..
  ROWSUM(irow)      =E= SUM(icol, SAM(irow,icol)) ;

*~~~~~~~~~ Define Coefficients
$ontext
 If a coeff is set it determines SAM.
 If ivalue sets SAM, then this equation determines coeff.
$offtext

 COEFFDEF(irow,icol)$((not izero(irow,icol)) and icolnz(icol))..
  COEFF(irow,icol)*COLSUM(icol)  =E= SAM(irow,icol) ;

*~~~~~~~~~ Row and column sum equality constraint
 SAMEQ(sacbal)..
  COLSUM(sacbal)     =E= ROWSUM(sacbal) ;

*~~~~~~~~~~ Macro-sam constraints
 MACSAMEQ(ssn,ssnp)$macmnz(ssn,ssnp)..
  MACSAM(ssn,ssnp) =E=
        SUM{(sacn,sacnp)$[map_ss_sac(ssn,sacn) and map_ss_sac(ssnp,sacnp)],
                         SAM(sacn,sacnp)} ;

 MACSUMEQ1(ssn)..
  MACSAM("m_sam_tot",ssn) =E= SUM(ss2np, MACSAM(ss2np,ssn)) ;

 MACSUMEQ2(ssn)..
  MACSAM(ssn,"m_sam_tot") =E= SUM(ss2np, MACSAM(ssn,ss2np)) ;

 MACSAMEQ2(ss,ssp)$MACSET(ss,ssp)..
           MACSAM(ss,ssp) =E= macsam0(ss,ssp) + ERR_MSAM(ss,ssp) ;

*~~~~~~~~~ Aggregate constraints

 AGGREQ(aggr2)..
  SUM((irow,icol), aggagg(irow,icol,aggr2)*SAM(irow,icol))
          =E= AGGTOTAL(aggr2) + ERR_AGG(aggr2) ;

* Ensure that exports do not exceed value of domestic production
 EXPORTEQ2(c)$SUM(w, P_SAM(c,w))..
  SUM(w, SAM(c,w)) =L= SUM(a, SAM(a,c)) + SUM(emrgn, SAM(emrgn,c)) +
                        SUM(etax, SAM(etax,c)) ;

*~~~~~~~~~ Definition of error terms

 ERROR_RC_EQ(icol2)$sigmay_RC(icol2,"col1")..
  ERR_RC(icol2,"col1") =E=
  SUM(jwt_RC, W_RC(icol2,"col1",jwt_RC)*vbar_RC(icol2,"col1",jwt_RC));

 SUMW_RC(icol2,rwcl)$sigmay_RC(icol2,rwcl)..
  SUM(jwt_RC, W_RC(icol2,rwcl,jwt_RC)) =E= 1 ;

 ERROR_R_RC_EQ(irow2)$sigmay_RC(irow2,"row1")..
  ERR_RC(irow2,"row1") =E=
  SUM(jwt_RC, W_RC(irow2,"row1",jwt_RC)*vbar_RC(irow2,"row1",jwt_RC));

 ERROR_AGG_EQ(aggr2)$sigmay_AGG(aggr2)..
  ERR_AGG(aggr2) =E= SUM(jwt_AGG, W_AGG(aggr2,jwt_AGG)*vbar_AGG(aggr2,jwt_AGG)) ;

 SUMW_AGG(aggr2)$sigmay_AGG(aggr2)..
  SUM(jwt_AGG, W_AGG(aggr2,jwt_AGG)) =E= 1 ;

 ERROR_TV_EQ(irow,icol)$((ivalue(irow,icol) or icoeff(irow,icol))
                      and sigmay_TV(irow,icol))..
  ERR_TV(irow,icol) =E= SUM(jwt_TV, W_TV(irow,icol,jwt_TV)*vbar_TV(irow,icol,jwt_TV)) ;

 SUMW_TV(irow,icol)$(estimate(irow,icol) and sigmay_TV(irow,icol))..
  SUM(jwt_TV, W_TV(irow,icol,jwt_TV)) =E= 1 ;

 ERROR_MSAM_EQ(ss,ssp)$(MACSET(ss,ssp) and sigmay_MSAM(ss,ssp))..
  ERR_MSAM(ss,ssp)  =E=
  SUM(jwt_MSAM, W_MSAM(ss,ssp,jwt_MSAM)*vbar_MSAM(ss,ssp,jwt_MSAM)) ;

 SUMW_MSAM(ss,ssp)$(MACSET(ss,ssp) and sigmay_MSAM(ss,ssp))..
  SUM(jwt_MSAM, W_MSAM(ss,ssp,jwt_MSAM)) =E= 1 ;

*~~~~~~~~~ Objective function
*$ONTEXT
 ENTROPY..
  DENTROPY =E=
    SUM((icol2,rwcl,jwt_RC)$sigmay_RC(icol2,rwcl),
       CENTROPY(W_RC(icol2,rwcl,jwt_RC),wbar_RC(icol2,rwcl,jwt_RC),epsilon))
                 +
    SUM((aggr2,jwt_AGG)$sigmay_AGG(aggr2),
       CENTROPY(W_AGG(aggr2,jwt_AGG),wbar_AGG(aggr2,jwt_AGG),epsilon))
                 +
    SUM((irow,icol,jwt_TV)$(ESTIMATE(irow,icol) and sigmay_TV(irow,icol)),
       CENTROPY(W_TV(irow,icol,jwt_TV),wbar_TV(irow,icol,jwt_TV),epsilon))
                 +
    SUM((ss,ssp,jwt_MSAM)$(MACSET(ss,ssp) and sigmay_MSAM(ss,ssp)),
       CENTROPY(W_MSAM(ss,ssp,jwt_MSAM),wbar_MSAM(ss,ssp,jwt_MSAM),epsilon))
     ;
*$OFFTEXT

$onText
 ENTROPY..
  DENTROPY =E= SUM((icol2,rwcl,jwt_RC)$sigmay_RC(icol2,rwcl),
                    W_RC(icol2,rwcl,jwt_RC)
                  * (LOG(W_RC(icol2,rwcl,jwt_RC) + epsilon)
                  - LOG(wbar_RC(icol2,rwcl,jwt_RC) + epsilon)))
                            +
               SUM((aggr2,jwt_AGG)$sigmay_AGG(aggr2),
                    W_AGG(aggr2,jwt_AGG)
                  * (LOG(W_AGG(aggr2,jwt_AGG) + epsilon)
                  - LOG(wbar_AGG(aggr2,jwt_AGG) + epsilon)))
                            +
               SUM((irow,icol,jwt_TV)$(estimate(irow,icol)
                                 and sigmay_TV(irow,icol)),
                    W_TV(irow,icol,jwt_TV)
                  * (LOG(W_TV(irow,icol,jwt_TV) + epsilon)
                  - LOG(wbar_TV(irow,icol,jwt_TV) + epsilon)))
                            +
               SUM((ss,ssp,jwt_MSAM)$(MACSET(ss,ssp)
                                 and sigmay_MSAM(ss,ssp)),
                    W_MSAM(ss,ssp,jwt_MSAM)
                  * (LOG(W_MSAM(ss,ssp,jwt_MSAM) + epsilon)
                  - LOG(wbar_MSAM(ss,ssp,jwt_MSAM) + epsilon)))

               ;
$offtext
*##################### END OF EQUATION ASSIGNMENT #######################

*######################## 15. DEFINE MODEL #################################

 MODEL SAMENTROP
 /
 SAMFLOW1
 SAMFLOW2
 SAMCOEFF1
 SAMCOEFF2
 COLSUMEQ1
 ROWSUMEQ1
 COLSUMDEF
 ROWSUMDEF
 COEFFDEF
 SAMEQ
 MACSAMEQ
 MACSUMEQ1
 MACSUMEQ2
 MACSAMEQ2
 AGGREQ
*CELLVAL
 EXPORTEQ2
 ERROR_RC_EQ
 ERROR_R_RC_EQ
 ERROR_AGG_EQ
 ERROR_TV_EQ
 ERROR_MSAM_EQ
 SUMW_RC
 SUMW_AGG
 SUMW_TV
 SUMW_MSAM
 ENTROPY
 / ;

*######################## END OF DEFINE MODEL ########################


*######################## 16. SOLVE MODEL #################################

 OPTION ITERLIM       = 10000 ;
* OPTION ITERLIM       = 1000000 ;
 OPTION RESLIM        = 100000 ;
 OPTION LIMROW        = 10, LIMCOL = 0;
 OPTION SOLPRINT      = ON ;

 SAMENTROP.TOLINFREP  = 0.001 ;
 SAMENTROP.TOLINFEAS  = 0.1 ;

 SAMENTROP.HOLDFIXED  = 1 ;
 OPTION NLP           = PATHNLP ;
* OPTION NLP           = CONOPT4 ;
* OPTION NLP           = CONOPT3 ;



* --- Solve statenment -------------------------------------------

 SOLVE SAMENTROP using nlp minimizing dentropy ;

*######################## END OF SOLVE ###############################

*####################### 17. PRELIMINARY REPORT ON RESULTS ##################

$INCLUDE 2_include\samest_anal.inc

*####################### END OF PRELIMINARY REPORT ON RESULTS ############

*########################## 18. EXPORTING RESULTS ###########################

* Sending the results to GDX

Execute_Unload  '3_output\%dstats%_sam_out.gdx',
                                  SEM,
                                  SAMCE,
                                  VALDIFF,
                                  PERDIFF,
                                  bigdiffv,
                                  bigdiffp,
                                  bigdiffvp,
                                  nbiggdiff,
                                  biggdiffcnt,
                                  DENTROPY.L,
                                  SAMBALCHK,
                                  samscal
                                  P_SAM
                                  MACSAM_EXOG
                                  MACSAM_ENDOG
                                  MACSAM_SOL
                                  
      ;


*########################## THE END #########################################
